# Automatically generated by pb2py
# fmt: off
if False:
    from typing_extensions import Literal

NORMAL = 0  # type: Literal[0]
BOLD = 1  # type: Literal[1]
MONO = 2  # type: Literal[2]
BR = 4  # type: Literal[4]
BR_HALF = 5  # type: Literal[5]
SET_COLOR = 6  # type: Literal[6]
