# Automatically generated by pb2py
# fmt: off
if False:
    from typing_extensions import Literal

TXINPUT = 0  # type: Literal[0]
TXOUTPUT = 1  # type: Literal[1]
TXMETA = 2  # type: Literal[2]
TXFINISHED = 3  # type: Literal[3]
TXEXTRADATA = 4  # type: Literal[4]
