# Automatically generated by pb2py
# fmt: off
if False:
    from typing_extensions import Literal

TIF_UNKNOWN = 0  # type: Literal[0]
GTE = 1  # type: Literal[1]
TIF_RESERVED = 2  # type: Literal[2]
IOC = 3  # type: Literal[3]
