# Automatically generated by pb2py
# fmt: off
if False:
    from typing_extensions import Literal

STAKE_REGISTRATION = 0  # type: Literal[0]
STAKE_DEREGISTRATION = 1  # type: Literal[1]
STAKE_DELEGATION = 2  # type: Literal[2]
