# Automatically generated by pb2py
# fmt: off
if False:
    from typing_extensions import Literal

SPENDADDRESS = 0  # type: Literal[0]
SPENDMULTISIG = 1  # type: Literal[1]
EXTERNAL = 2  # type: Literal[2]
SPENDWITNESS = 3  # type: Literal[3]
SPENDP2SHWITNESS = 4  # type: Literal[4]
