# Automatically generated by pb2py
# fmt: off
if False:
    from typing_extensions import Literal

Bip39 = 0  # type: Literal[0]
Slip39_Basic = 1  # type: Literal[1]
Slip39_Advanced = 2  # type: Literal[2]
