# Automatically generated by pb2py
# fmt: off
if False:
    from typing_extensions import Literal

Strict = 0  # type: Literal[0]
PromptAlways = 1  # type: Literal[1]
PromptTemporarily = 2  # type: Literal[2]
