# Automatically generated by pb2py
# fmt: off
if False:
    from typing_extensions import Literal

Bitcoin = 1  # type: Literal[1]
Bitcoin_like = 2  # type: Literal[2]
Binance = 3  # type: Literal[3]
Cardano = 4  # type: Literal[4]
Crypto = 5  # type: Literal[5]
EOS = 6  # type: Literal[6]
Ethereum = 7  # type: Literal[7]
Lisk = 8  # type: Literal[8]
Monero = 9  # type: Literal[9]
NEM = 10  # type: Literal[10]
Ripple = 11  # type: Literal[11]
Stellar = 12  # type: Literal[12]
Tezos = 13  # type: Literal[13]
U2F = 14  # type: Literal[14]
Shamir = 15  # type: Literal[15]
ShamirGroups = 16  # type: Literal[16]
PassphraseEntry = 17  # type: Literal[17]
