# Automatically generated by pb2py
# fmt: off
if False:
    from typing_extensions import Literal

Other = 1  # type: Literal[1]
FeeOverThreshold = 2  # type: Literal[2]
ConfirmOutput = 3  # type: Literal[3]
ResetDevice = 4  # type: Literal[4]
ConfirmWord = 5  # type: Literal[5]
WipeDevice = 6  # type: Literal[6]
ProtectCall = 7  # type: Literal[7]
SignTx = 8  # type: Literal[8]
FirmwareCheck = 9  # type: Literal[9]
Address = 10  # type: Literal[10]
PublicKey = 11  # type: Literal[11]
MnemonicWordCount = 12  # type: Literal[12]
MnemonicInput = 13  # type: Literal[13]
_Deprecated_ButtonRequest_PassphraseType = 14  # type: Literal[14]
UnknownDerivationPath = 15  # type: Literal[15]
RecoveryHomepage = 16  # type: Literal[16]
Success = 17  # type: Literal[17]
Warning = 18  # type: Literal[18]
PassphraseEntry = 19  # type: Literal[19]
PinEntry = 20  # type: Literal[20]
