# Automatically generated by pb2py
# fmt: off
if False:
    from typing_extensions import Literal

UP = 0  # type: Literal[0]
DOWN = 1  # type: Literal[1]
LEFT = 2  # type: Literal[2]
RIGHT = 3  # type: Literal[3]
