# Automatically generated by pb2py
# fmt: off
if False:
    from typing_extensions import Literal

ImportanceTransfer_Activate = 1  # type: Literal[1]
ImportanceTransfer_Deactivate = 2  # type: Literal[2]
