# Automatically generated by pb2py
# fmt: off
if False:
    from typing_extensions import Literal

OT_UNKNOWN = 0  # type: Literal[0]
MARKET = 1  # type: Literal[1]
LIMIT = 2  # type: Literal[2]
OT_RESERVED = 3  # type: Literal[3]
