# Automatically generated by pb2py
# fmt: off
if False:
    from typing_extensions import Literal

UnexpectedMessage = 1  # type: Literal[1]
ButtonExpected = 2  # type: Literal[2]
DataError = 3  # type: Literal[3]
ActionCancelled = 4  # type: Literal[4]
PinExpected = 5  # type: Literal[5]
PinCancelled = 6  # type: Literal[6]
PinInvalid = 7  # type: Literal[7]
InvalidSignature = 8  # type: Literal[8]
ProcessError = 9  # type: Literal[9]
NotEnoughFunds = 10  # type: Literal[10]
NotInitialized = 11  # type: Literal[11]
PinMismatch = 12  # type: Literal[12]
WipeCodeMismatch = 13  # type: Literal[13]
FirmwareError = 99  # type: Literal[99]
