# Automatically generated by pb2py
# fmt: off
if False:
    from typing_extensions import Literal

PAYTOADDRESS = 0  # type: Literal[0]
PAYTOSCRIPTHASH = 1  # type: Literal[1]
PAYTOMULTISIG = 2  # type: Literal[2]
PAYTOOPRETURN = 3  # type: Literal[3]
PAYTOWITNESS = 4  # type: Literal[4]
PAYTOP2SHWITNESS = 5  # type: Literal[5]
