# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p

if __debug__:
    try:
        from typing import Dict, List  # noqa: F401
        from typing_extensions import Literal  # noqa: F401
    except ImportError:
        pass


class EosAuthorizationKey(p.MessageType):

    def __init__(
        self,
        type: int = None,
        key: bytes = None,
        address_n: List[int] = None,
        weight: int = None,
    ) -> None:
        self.type = type
        self.key = key
        self.address_n = address_n if address_n is not None else []
        self.weight = weight

    @classmethod
    def get_fields(cls) -> Dict:
        return {
            1: ('type', p.UVarintType, 0),
            2: ('key', p.BytesType, 0),
            3: ('address_n', p.UVarintType, p.FLAG_REPEATED),
            4: ('weight', p.UVarintType, 0),
        }
