# Automatically generated by pb2py
# fmt: off
if False:
    from typing_extensions import Literal

Transfer = 0  # type: Literal[0]
RegisterSecondPassphrase = 1  # type: Literal[1]
RegisterDelegate = 2  # type: Literal[2]
CastVotes = 3  # type: Literal[3]
RegisterMultisignatureAccount = 4  # type: Literal[4]
CreateDapp = 5  # type: Literal[5]
TransferIntoDapp = 6  # type: Literal[6]
TransferOutOfDapp = 7  # type: Literal[7]
