# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p

if __debug__:
    try:
        from typing import Dict, List  # noqa: F401
        from typing_extensions import Literal  # noqa: F401
        EnumTypeInputScriptType = Literal[0, 1, 2, 3, 4]
    except ImportError:
        pass


class AuthorizeCoinJoin(p.MessageType):
    MESSAGE_WIRE_TYPE = 51

    def __init__(
        self,
        coordinator: str = None,
        max_total_fee: int = None,
        fee_per_anonymity: int = None,
        address_n: List[int] = None,
        coin_name: str = None,
        script_type: EnumTypeInputScriptType = None,
    ) -> None:
        self.coordinator = coordinator
        self.max_total_fee = max_total_fee
        self.fee_per_anonymity = fee_per_anonymity
        self.address_n = address_n if address_n is not None else []
        self.coin_name = coin_name
        self.script_type = script_type

    @classmethod
    def get_fields(cls) -> Dict:
        return {
            1: ('coordinator', p.UnicodeType, 0),  # required
            2: ('max_total_fee', p.UVarintType, 0),  # required
            3: ('fee_per_anonymity', p.UVarintType, 0),
            4: ('address_n', p.UVarintType, p.FLAG_REPEATED),
            5: ('coin_name', p.UnicodeType, 0),  # default=Bitcoin
            6: ('script_type', p.EnumType("InputScriptType", (0, 1, 2, 3, 4)), 0),  # default=SPENDADDRESS
        }
