# Automatically generated by pb2py
# fmt: off
if False:
    from typing_extensions import Literal

CosignatoryModification_Add = 1  # type: Literal[1]
CosignatoryModification_Delete = 2  # type: Literal[2]
