# Automatically generated by pb2py
# fmt: off
if False:
    from typing_extensions import Literal

BASE = 0  # type: Literal[0]
BASE_SCRIPT_KEY = 1  # type: Literal[1]
BASE_KEY_SCRIPT = 2  # type: Literal[2]
BASE_SCRIPT_SCRIPT = 3  # type: Literal[3]
POINTER = 4  # type: Literal[4]
POINTER_SCRIPT = 5  # type: Literal[5]
ENTERPRISE = 6  # type: Literal[6]
ENTERPRISE_SCRIPT = 7  # type: Literal[7]
BYRON = 8  # type: Literal[8]
REWARD = 14  # type: Literal[14]
REWARD_SCRIPT = 15  # type: Literal[15]
