# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p

from .EosPermissionLevel import EosPermissionLevel

if __debug__:
    try:
        from typing import Dict, List  # noqa: F401
        from typing_extensions import Literal  # noqa: F401
    except ImportError:
        pass


class EosActionCommon(p.MessageType):

    def __init__(
        self,
        account: int = None,
        name: int = None,
        authorization: List[EosPermissionLevel] = None,
    ) -> None:
        self.account = account
        self.name = name
        self.authorization = authorization if authorization is not None else []

    @classmethod
    def get_fields(cls) -> Dict:
        return {
            1: ('account', p.UVarintType, 0),
            2: ('name', p.UVarintType, 0),
            3: ('authorization', EosPermissionLevel, p.FLAG_REPEATED),
        }
